      program extract
*****************************************************************
*                                                               *
*               AUTHOR: G. WOTAWA                               *
*               1994-05-04                                      *
*               Update: 1996-09-23 (Migration to VPP700)        *
*               Update: 1998-07-14                              *
*                                                               *
*        ******************************************             *
*        * W O R K S T A T I O N    V E R S I O N *             *
*        ******************************************             *
*                                                               *
*****************************************************************
*                                                               *
* DESCRIPTION: THIS PROGRAM CREATES (subroutine create_job) AND *
* EXECUTES (subroutine execute_job) A JOB DESIGNED TO PREPARE   *
* METEOROLOGICAL DATA FOR TRAJECTORY MODEL FLEXTRA OR LAGRANGIAN*
* PARTICLE DISPERSION MODEL FLEXPART.                           *
* THE VERTICAL VELOCITY ETAPOINT IS CALCULATED.                 *
* AFTERWARDS, DATA FILES ARE SAVED IN AN ECFILE CATALOG.        *
*                                                               *
* DOCUMENTATION:                                                *
*                                                               *
* http://www.boku.ac.at/imp/envmet/ecmwf_extr.html              *
*                                                               *
*****************************************************************
*                                                               *
* JOB CONTROL FILE: CONTROL                                     *
*                                                               *
* 1.   BEGINNING DATE [yyyymmddhh]                              *
* 2.   ENDING DATE    [yyyymmddhh]                              *
* 3.   TIME INTERVAL  [hr]                                      *
* 4.   GRID RESOLUTION [DEG]                                    *
* 5.   GRID DOMAIN - LLON/RLON/BLAT/TLAT                        *
* 6.   VERTICAL LAYERS TO EXTRACT                               *
* 7.   VERTICAL LAYERS AVAILABLE FROM ECMWF                     *
* 8.   SPECTRAL TRUNCATION NEEDED                               *
*                                                               *
*****************************************************************
*                                                               *
* DATA RETRIEVED FROM MARS ARCHIVE, ASSOCIATED FORTRAN UNIT     *
*                                                               *
*                     U,V,T,Q  fort.11                          *
*                     D     fort.12                             *
*                     LNSP  fort.13                             *
*                     SURFACE DATA fort.14                      *
*                                                               *
* FLUX DATA: RETRIEVAL FROM ECFILE CATALOG TO fort.16           * 
*                                                               *
* PLEASE NOTICE:                                                *
*                                                               *
* PRECALCULATION OF DEACCUMULATED FLUX DATA IS NECCESSARY BEFORE*
* THE EXECUTION OF THIS PROGRAM IN PREPROCESSOR MODE !!!!       *
*                                                               *
*****************************************************************
*                                                               *
* EXECUTION OF ETAPOINT CALCULATION PROGRAM                     *
* CONVERT_PRE                                                   *
*                                                               *
* SOURCE-FILES DO EXIST IN THE FOLLOWING CATALOGS:              *
*                                                               *
* ~ka7/preconvert.qsh                                           *
* ~ka7/preconvert.qgg                                           *
*                                                               *
* INCLUDE-FILES: ARE GENERATED AUTOMATICALLY ACCORDING TO GIVEN *
*                USER SPECIFICATIONS                            *
*                                                               *
* OUTPUT FILE: fort.15                                          *
*           SAVED AS ENYYMMDDHH IN SPECIFIED ECFILE DIRECTORY   *
*                                                               *
*****************************************************************

      implicit none

      integer jmd1,jmd2,hms1,hms2,idel,ijmd,ih,nuvz,nwz
      integer jmd,hms,jmdupdate,hmsupdate,jmd3dvar,hms3dvar
      integer nlev_ec,ktrunc
      character uname*3,instdir*80,fluxdir*80,metedir*80
      real resol,llon,rlon,blat,tlat
      logical update,updateold,firstcall,new3dvar
      double precision juldate,jul,jul1,jul2,deljul,julupdate
      double precision jul3dvar

      data jmdupdate/19950404/,hmsupdate/0/
      data jmd3dvar/19960130/,hms3dvar/0/
      data update /.false./,updateold/.false./,
     &     firstcall/.true./,new3dvar/.false./
      julupdate=juldate(jmdupdate,hmsupdate)
      jul3dvar=juldate(jmd3dvar,hms3dvar)

*************************
* READ JOB CONTROL FILE *
*******************************************************************
      open(102,file='CONTROL',status='old',err=999)
      read(102,5)  jmd1,hms1  ! BEGINNING DATE (YYYYMMDDHH)
      read(102,5)  jmd2,hms2  ! ENDING DATE (YYYYMMDDHH)
      read(102,10) idel       ! TIME SHIFT (HRS)
      read(102,*) resol       ! GRID RESOLUTION (DEG)
      read(102,*) llon,rlon,blat,tlat  ! AREA
      read(102,*) nuvz,nwz    ! NUMBER OF LEVELS
      read(102,*) nlev_ec     ! NUMBER OF LEVELS ECMWF
      read(102,*) ktrunc      ! SPECTRAL TRUNCATION
      read(102,'(a)') uname   ! USER NAME
      read(102,'(a)') instdir ! INSTALLATION DIRECTORY
      read(102,'(a)') fluxdir ! SCRATCH DIRECTORY FLUX INPUT DATA
      read(102,'(a)') metedir ! ECFILE DIRECTORY OUTPUT DATA
      close(102)
*******************************************************************

5     format(i8,i2)
10    format(i2)
15    format(i1)

      hms1=hms1*10000
      hms2=hms2*10000
      jul1=juldate(jmd1,hms1)
      jul2=juldate(jmd2,hms2)
      deljul=dble(float(idel)/24.)
      if(jul1.gt.jul2) 
     &  stop ' BEGINNING DATE GREATER THEN ENDING DATE - STOP'

* DATE/TIME LOOP

      do 50 jul=jul1,jul2,deljul
        call caldate(jul,jmd,hms)
        if(jul.ge.julupdate) update=.true.   ! CHANGES IN MARS ARCHIVE
        if(update.eq..true..and.updateold.eq..false.)
     &    firstcall=.true.
        if(jul.ge.jul3dvar) new3dvar=.true.  ! INTRODUCTION OF 3D-VAR
C        ijmd=jmd-19000000
        ijmd=jmd
        ih=hms/10000

* APPEND CURRENT DATE TO JOBFILE

        call create_job(ijmd,ih,resol,llon,rlon,blat,tlat,nuvz,nwz,
     &    nlev_ec,ktrunc,firstcall,update,new3dvar,uname,instdir,
     &    fluxdir,metedir)
        if(firstcall) firstcall=.false.
        updateold=update
50    continue
      call execute_job(uname)                ! EXECUTE JOB (BATCH)
      stop 'FINNISHED PROGRAMM EXECUTION'
999   write(*,*) ' NOTICE --> FILE "CONTROL" NOT AVAILABLE '
1000  end
