      PROGRAM PRECONVERT
*******************************************************************
*                                                                 *
* PROGRAM PRECONVERT - PREPARES INPUT DATA FOR POP MODEL METEOR-  *
*                      OLOGICAL PREPROCESSOR                      *
*                                                                 *
*          ******************************************             *
*          * W O R K S T A T I O N    V E R S I O N *             *
*          ******************************************             *
*                                                                 *
*          NEW UPDATE: 14/07/1998 by G.W.                         *
*                                                                 *
*           --> Double precision introduced to most parts         *
*               of the code to avoid numerical errors             *
*           --> Avoids problems for phi=90 degrees (Pole)         *
*                                                                 *
*******************************************************************
*                                                                 *
* CALCULATION OF ETAPOINT ON A REGULAR LAMDA/PHI GRID AND WRITING *
* U,V,ETAPOINT,T,PS,Q,SD,MSL,TCC,10U, 10V, 2T,2D,LSP,CP,SSHF,SSR, *
* EWSS,NSSS TO AN OUTPUT FILE (GRIB FORMAT).                      * 
*                                                                 *
* AUTHORS: L. HAINBERGER, G. WOTAWA, 1994-04                      *
*                                                                 *
*******************************************************************
*                                                                 *
* DESCRIPTION OF NEEDED INPUT:                                    *
*                                                                 *
* UNIT  FILE      PARAMETER(S)    DATA REPRESENTATION             *
*                                                                 *
* 11    fort.11   T,U,V,Q         regular lamda phi grid          *
* 12    fort.12   D               regular lamda phi grid          *
* 13    fort.13   LNSP            spherical harmonics             *
* 14    fort.14   SD,MSL,TCC,10U,                                 *
*                 10V,2T,2D       regular lamda phi grid          *
* 16    fort.16   LSP,CP,SSHF,                                    *
*                 SSR,EWSS,NSSS   regular lamda phi grid          *
*                                                                 *
*******************************************************************
*                                                                 *
* DESCRIPTION OF OUTPUT:                                          *
*                                                                 *
* UNIT  FILE      PARAMETER(S)    DATA REPRESENTATION             *
*                                                                 *
* 15    fort.15   U,V,ETA,T,PS,                                   *
*                 Q,SD,MSL,TCC,                                   *
*                 10U,10V,2T,2D,  regular lamda phi grid          *
*                 LSP,CP,SSHF,                                    *
*                 SSR,EWSS,NSSS                                   *
*                                                                 *
*******************************************************************
*                                                                 *
* INCLUDE FILE FOR PARAMETER STATEMENTS - CHANGE IF NECCESSARY    *
*                                                                 *
*******************************************************************
*
      include 'includepar'

*******************************************************************
*                       VARIABLE DECLARATION                      *
*******************************************************************

      REAL*8 LNPS(0:(MNAUF+1)*(MNAUF+2)-1,1)
      REAL*8 LNPSL(0:(MNAUF+1)*(MNAUF+2)-1,1)
      REAL*8 LNPSM(0:(MNAUF+1)*(MNAUF+2)-1,1)
      
      REAL*8 Z(0:((MNAUF+1)*(MNAUF+2))/2,MAXB)
      REAL T(MAXL,MAXB,MLEVEL),U(MAXL,MAXB,MLEVEL),V(MAXL,MAXB,MLEVEL)
      REAL QA(MAXL,MAXB,MLEVEL)
      REAL UMS(MAXL,MAXB,MLEVEL),VMS(MAXL,MAXB,MLEVEL)
      REAL*8 DIV(MAXL,MAXB,MLEVEL),ETA(MAXL,MAXB,MLEVEL)
      REAL*8 DPSDL(MAXL,MAXB),DPSDM(MAXL,MAXB)
      REAL*8 PS(MAXL,MAXB)
      REAL*8 UR8(MAXL,MAXB,MLEVEL),VR8(MAXL,MAXB,MLEVEL)
      REAL PSURF(MAXL,MAXB),ETASNGL(MAXL,MAXB,MLEVEL)

      REAL SD(MAXL,MAXB),MSL(MAXL,MAXB),TCC(MAXL,MAXB)
      REAL U10(MAXL,MAXB),V10(MAXL,MAXB)
      REAL T2(MAXL,MAXB),D2(MAXL,MAXB)

      REAL LSP(MAXL,MAXB),CP(MAXL,MAXB),SSHF(MAXL,MAXB)
      REAL SSR(MAXL,MAXB),EWSS(MAXL,MAXB),NSSS(MAXL,MAXB)
      
      REAL*8 WSAVE(4*MAUF*2+15)
      REAL*8 BREITE(MAXB),AK(MLEVEL+1),BK(MLEVEL+1)
      REAL PI,COSB
      REAL*8 URLAR8,JMIN1,LLLAR8,MAXBMIN1,PIR8,DCOSB

*******************************************************************
*     PARAMETERS,VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING    *
*******************************************************************

      INTEGER I,J,K,L,JPACK,JPUNP,IFIELD,IWORD
      INTEGER INDE,LUNIT,LENGHT,levtotal,LEV

      PARAMETER (JPACK=4*MAXL*MAXB)
      PARAMETER (JPUNP=4*JPACK)

      INTEGER ISEC0(2),JSEC0(2)
      INTEGER ISEC1(56),JSEC1(56)
      INTEGER ISEC2(22+MAXL+MAXB),JSEC2(22+MAXL+MAXB)
      INTEGER ISEC3(2),JSEC3(2)
      INTEGER ISEC4(64),JSEC4(64)
      
      REAL ZSEC2(60+2*MLEVEL),YSEC2(60+2*MLEVEL)
      REAL ZSEC3(2),YSEC3(2)
      
      INTEGER INBUFF(JPACK)
      REAL ZSEC4(JPUNP)

      CHARACTER*1 YOPER
      integer ierr
      
*******************************************************************
*INITIALIZATION OF FAST FOURIER TRANSFORMATION AND LEGENDRE FIELDS*
*******************************************************************

      IF(MAUF.LT.2*MNAUF) THEN
         CALL CFFTI(2*MAUF,WSAVE)
      ELSE
         CALL CFFTI(MAUF,WSAVE)
      ENDIF
			
      PI=4.*ATAN(1.0)
      DO 20 J=1,MAXB

      URLAR8=DBLE(URLA)
      LLLAR8=DBLE(LLLA)
      JMIN1=DBLE(FLOAT(J-1))
      MAXBMIN1=DBLE(FLOAT(MAXB-1))
      PIR8=DBLE(PI)

      BREITE(J)=DSIN((URLAR8-JMIN1*(URLAR8-LLLAR8)/MAXBMIN1)*
     &  PIR8/180.0)

      CALL PLGNDN(MNAUF,BREITE(J),Z(0,J))

20    CONTINUE

*******************************************************************
*          READING OF PREPARED METEOROLOGICAL FIELDS              *
*                                                                 *
*          THE FOLLOWING FIELDS ARE EXPECTED:                     *
*                                                                 *
*          UNIT 11: T,U,V,Q      (REGULAR GRID)                   *
*          UNIT 13: D            (REGULAR GRID)                   *
*          UNIT 12: LNSP         (SPHERICAL HARMONICS)            *
*          UNIT 14: SURFACE DATA (REGULAR GRID)                   *
*          UNIT 16: FLUX DATA    (REGULAR GRID)                   *
*******************************************************************
*
*******************************************************************
*                      READING OF U,V AND T                       *
*******************************************************************
*
* OPENING OF UNBLOCKED GRIB FILE
*
      call PBOPEN(LUNIT,'fort.11','r',IERR)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING fort.11'
      ifield=0
 50   ifield=ifield+1
*
*     Get next fields
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.eq.-1) goto 200   ! EOF DETECTED
      if(ierr.lt.-1) goto 3000  ! ERROR DETECTED

      YOPER = 'D'
      IERR = 1
*
* PARAMETERS NEEDED FOR ROUTINE GRIBEX: SEE DESCRIPTION
*
      CALL GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     C             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
*
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      IF(IFIELD.GT.4*MLEVEL) THEN
         write(*,*) 'PROGRAMM PRECONVERT - ERROR'
         write(*,*) 'NUMBER OF VERTICAL LEVELS EXPECTED: ',mlevel
         write(*,*) 'TRIED TO READ MORE LEVELS ...'
         stop
      endif

      LEV=ISEC1(8)
      L=1
      DO 312 J=1,MAXB
      COSB=SQRT(1.0-SNGL(BREITE(J))*SNGL(BREITE(J)))
      DCOSB=DSQRT(1.0d0-BREITE(J)*BREITE(J))
      DO 312 I=1,MAXL
         IF (ISEC1(6).eq.130) T  (I,J,LEV)=ZSEC4(L)
         IF (ISEC1(6).eq.131) U  (I,J,LEV)=ZSEC4(L)*COSB
         IF (ISEC1(6).eq.131) UR8(I,J,LEV)=DBLE(ZSEC4(L))*DCOSB
         IF (ISEC1(6).eq.131) UMS(I,J,LEV)=ZSEC4(L)
         IF (ISEC1(6).eq.132) V  (I,J,LEV)=ZSEC4(L)*COSB
         IF (ISEC1(6).eq.132) VR8(I,J,LEV)=DBLE(ZSEC4(L))*DCOSB
         IF (ISEC1(6).eq.132) VMS(I,J,LEV)=ZSEC4(L)
         IF (ISEC1(6).eq.133) QA (I,J,LEV)=ZSEC4(L)
         L=L+1
312   CONTINUE
      GO TO 50
200   CALL PBCLOSE(LUNIT,IERR)

* CREATE FILE VERTICAL.EC NEEDED BY POP MODEL

      levtotal=isec2(12)/2-1
      if(levtotal.ne.mlevel) then
         write(*,*) 'PROGRAMM PRECONVERT - ERROR'
         write(*,*) 'NUMBER OF ECMWF MODEL LEVELS: ',levtotal
         write(*,*) 'NUMBER OF EXPECTED LEVELS:    ',mlevel
         write(*,*) 'PLEASE CHANGE PARAMETER STATEMENT'
         stop
      endif
      open(21,file='VERTICAL.EC')
      write(21,'(a)')
      write(21,'(a)') 'VERTICAL DISCRETIZATION OF POP MODEL'
      write(21,'(a)')
      write(21,'(i3,a)') levtotal,'   number of layers'
      write(21,'(a)')
      write(21,'(a)') '* A(NLEV+1)'
      write(21,'(a)')
      do 205 i=1,levtotal+1
         AK(I)=DBLE(ZSEC2(10+i))
205      write(21,'(f18.12)') zsec2(10+i)
      write(21,'(a)')
      write(21,'(a)') '* B(NLEV+1)'
      write(21,'(a)')
      J=0
      do 210 i=levtotal+2,2*(levtotal+1)
         J=J+1
         BK(J)=DBLE(ZSEC2(10+i))
210      write(21,'(f18.12)') zsec2(10+i)
      close(21)

*******************************************************************
*                     READING OF DIVERGENCE                       *
*******************************************************************
*
* OPENING OF UNBLOCKED GRIB FILE
*
      call PBOPEN(LUNIT,'fort.13','r',IERR)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING fort.13'
       
      ifield=0
60    ifield=ifield+1
*
*     Get next fields
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.eq.-1) goto 220   ! EOF DETECTED
      if(ierr.lt.-1) goto 3000  ! ERROR DETECTED

      YOPER = 'D'
      IERR = 1
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'

      IF(IFIELD.GT.MLEVEL) THEN
         write(*,*) 'PROGRAMM PRECONVERT - ERROR'
         write(*,*) 'NUMBER OF VERTICAL LEVELS EXPECTED: ',mlevel
         write(*,*) 'TRIED TO READ MORE LEVELS ...'
         stop
      endif

      L=1
      DO 313 J=1,MAXB
      DO 313 I=1,MAXL
         DIV(I,J,IFIELD)=DBLE(ZSEC4(L))
         L=L+1
313   CONTINUE

      GO TO 60
220   CALL PBCLOSE(LUNIT,IERR)

*******************************************************************
*                  READING OF SURFACE PRESSURE                    *
*******************************************************************
*
* OPENING OF UNBLOCKED GRIB FILE
*
      call PBOPEN(LUNIT,'fort.12','r',IERR)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING fort.12'

      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
C                                                                               
      IF (IERR.NE.0) GOTO 9999
      YOPER = 'D'
      IERR = 1
      CALL GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     C             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      IF(ISEC2(2).ne.MNAUF.or.ISEC2(3).ne.MNAUF.or.
     &   ISEC2(4).ne.MNAUF) then
         write(*,*) 'PROGRAMM PRECONVERT - ERROR'
         write(*,*) 'SPECTRAL RESOLUTION DETECTED: ',ISEC2(2)
         write(*,*) '                              ',ISEC2(3)
         write(*,*) '                              ',ISEC2(4)
         write(*,*) 'SPECTRAL RESOLUTION EXPECTED: ',MNAUF
         write(*,*) 'PLEASE CHANGE PARAMETER STATEMENT'
         stop
      endif

      K=1
      DO 5 I=0,MNAUF
         INDE=((I+1)*I)/2
         DO 9 J=I,MNAUF
            INDE=INDE+J
            LNPS(2*INDE,1)=DBLE(ZSEC4(K))
            LNPS(2*INDE+1,1)=DBLE(ZSEC4(K+1))
            K=K+2
9        CONTINUE
5     CONTINUE
      CALL PBCLOSE(LUNIT,IERR)
*
*******************************************************************
*                  READING OF SURFACE DATA                        *
*******************************************************************
*
* OPENING OF UNBLOCKED GRIB FILE
*
      call PBOPEN(LUNIT,'fort.14','r',IERR)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING fort.14'

      ifield=0
240   ifield=ifield+1
*
*     Get next fields
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.eq.-1) goto 250   ! EOF DETECTED
      if(ierr.lt.-1) goto 3000  ! ERROR DETECTED

      YOPER = 'D'
      IERR = 1
      CALL GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     C             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'

      L=1
      DO 319 J=1,MAXB
      DO 319 I=1,MAXL
         IF(ISEC1(6).eq.141) SD (I,J)=ZSEC4(L)
         IF(ISEC1(6).eq.151) MSL(I,J)=ZSEC4(L)
         IF(ISEC1(6).eq.164) TCC(I,J)=ZSEC4(L)
         IF(ISEC1(6).eq.165) U10(I,J)=ZSEC4(L)
         IF(ISEC1(6).eq.166) V10(I,J)=ZSEC4(L)
         IF(ISEC1(6).eq.167) T2 (I,J)=ZSEC4(L)
         IF(ISEC1(6).eq.168) D2 (I,J)=ZSEC4(L)
         L=L+1
319   CONTINUE
      GOTO 240
250   CALL PBCLOSE(LUNIT,IERR)
*
*******************************************************************
*                  READING OF DEACCUMULATED FLUX DATA             *
*******************************************************************

*
* OPENING OF UNBLOCKED GRIB FILE
*
      call PBOPEN(LUNIT,'fort.16','r',IERR)
      if(ierr.lt.0) THEN
         WRITE(*,*) ' ++++++++++++ PLEASE NOTICE: ++++++++++++++'
         WRITE(*,*) ' CONVERT_PRE: NO FLUX DATA FOUND IN ECDIR  '
         WRITE(*,*) ' NO FLUXES WILL BE STORED IN EN-FILES      '
         WRITE(*,*) ' !! THIS IS A PROBLEM FOR FLEXPART USERS !!'
         WRITE(*,*) ' ++++++++++++++++++++++++++++++++++++++++++'
         GOTO 222
      ENDIF
*
* READING AND DECODING LSP
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.ne.0) stop 'SORRY: WRONG FILE FORMAT !'
      call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      if(isec1(6).ne.142) stop 'SORRY: WRONG FILE FORMAT !'

      L=1
      do 500 J=1,MAXB
         do 500 I=1,MAXL
            LSP(i,j)=ZSEC4(L)
            L=L+1
500   CONTINUE
*
* READING AND DECODING CP
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.ne.0) stop 'SORRY: WRONG FILE FORMAT !'
      call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      if(isec1(6).ne.143) stop 'SORRY: WRONG FILE FORMAT !'
 
      L=1
      do 600 J=1,MAXB
         do 600 I=1,MAXL
            CP(i,j)=ZSEC4(L)
            L=L+1
600   CONTINUE
*
* READING AND DECODING SSHF
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.ne.0) stop 'SORRY: WRONG FILE FORMAT !'
      call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      if(isec1(6).ne.146) stop 'SORRY: WRONG FILE FORMAT !'
 
      L=1
      do 700 J=1,MAXB
         do 700 I=1,MAXL
            SSHF(i,j)=ZSEC4(L)
            L=L+1
700   CONTINUE
*
* READING AND DECODING SSR
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.ne.0) stop 'SORRY: WRONG FILE FORMAT !'
      call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      if(isec1(6).ne.176) stop 'SORRY: WRONG FILE FORMAT !'
 
      L=1
      do 800 J=1,MAXB
         do 800 I=1,MAXL
            SSR(i,j)=ZSEC4(L)
            L=L+1
800   CONTINUE
*
* READING AND DECODING EWSS
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.ne.0) stop 'SORRY: WRONG FILE FORMAT !'
      call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      if(isec1(6).ne.180) stop 'SORRY: WRONG FILE FORMAT !'

      L=1
      do 900 J=1,MAXB
         do 900 I=1,MAXL
            EWSS(i,j)=ZSEC4(L)
            L=L+1
900   CONTINUE
*
* READING AND DECODING NSSS
*
      call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.ne.0) stop 'SORRY: WRONG FILE FORMAT !'
      call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &             ZSEC4,JPUNP,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SORRY: GRIB CODE CONVERSION ERROR'
      if(isec1(6).ne.181) stop 'SORRY: WRONG FILE FORMAT !'
 
      L=1
      do 1000 J=1,MAXB
         do 1000 I=1,MAXL
            NSSS(i,j)=ZSEC4(L)
            L=L+1
1000  CONTINUE
*
* CLOSING OF INPUT DATA FILE fort.16
* 
      CALL PBCLOSE(LUNIT,IERR)

222   CONTINUE
*
* FINNISHED READING OF MARS DATA
*
      CALL PHGRPH(LNPS,LNPSL,LNPSM,MNAUF,1)

      CALL PHGCUT(LNPS,PS,WSAVE,Z,
     *            MNAUF,MAUF,MANF,MAXL,MAXB,1)
     
      DO 333 I=1,MAXL
         DO 333 J=1,MAXB
            PS(I,J)=DEXP(PS(I,J))
            PSURF(I,J)=SNGL(PS(I,J))
333   CONTINUE
      
      CALL PHGCUT(LNPSL,DPSDL,WSAVE,Z,
     *MNAUF,MAUF,MANF,MAXL,MAXB,1)

      CALL PHGCUT(LNPSM,DPSDM,WSAVE,Z,
     *MNAUF,MAUF,MANF,MAXL,MAXB,1)

*******************************************************************
*      CALCULATION OF ETAPOINT --> TOTAL TIME DERIVATIVE OF       *
*      ECMWF VERTICAL COORDINATE ETA MULTIPLIED BY DERIVATIVE     *
*      OF PRESSURE IN ETA DIRECTION                               *
*******************************************************************
    
      CALL CONTGL(PS,DPSDL,DPSDM,DIV,UR8,VR8,BREITE,ETA,MAXL,MAXB,
     &  MLEVEL,AK,BK)

      DO 444 I=1,MAXL
        DO 444 J=1,MAXB
          DO 444 K=1,MLEVEL
444         ETASNGL(I,J,K)=SNGL(ETA(I,J,K))

      IF(ABS(BREITE(1)-1.0d0).LT.1.0d-15) THEN
        WRITE(*,*) 'NOTICE: NORTH-POLE INCLUDED IN FIELD'
        WRITE(*,*) 'SET W(90) TO W(89.X)'

* NORTH POLE INCLUDED: AVOID 90 DEG/POLE ERROR

        DO 555 I=1,MAXL
          DO 555 K=1,MLEVEL
555         ETASNGL(I,1,K)=ETASNGL(I,2,K)

      ENDIF

      IF(ABS(BREITE(MAXB)+1.0d0).LT.1.0d-15) THEN
        WRITE(*,*) 'NOTICE: SOUTH-POLE INCLUDED IN FIELD'
        WRITE(*,*) 'SET W(-90) TO W(-89.X)'

* SOUTH POLE INCLUDED: AVOID -90 DEG/POLE ERROR

        DO 666 I=1,MAXL
          DO 666 K=1,MLEVEL
666         ETASNGL(I,MAXB,K)=ETASNGL(I,MAXB-1,K)

      ENDIF

*******************************************************************
*   WRITE U,V,ETAPOINT,T AND PSURF TO OUTPUT FILE (GRIB FORMAT)   *
*******************************************************************

      CALL WRITEOUT_PRE(UMS,VMS,ETASNGL,T,QA,PSURF,SD,MSL,TCC,U10,V10,
     &              T2,D2,LSP,CP,SSHF,SSR,EWSS,NSSS,JSEC0,JSEC1,
     &              JSEC2,JSEC3,JSEC4,YSEC2,YSEC3,ZSEC4,JPUNP,
     &              INBUFF,JPACK)

      STOP 'SUCCESSFULLY FINNISHED CONVERT_PRE: CONGRATULATIONS'
 3000 STOP 'ROUTINE CONVERT_PRE: ERROR'
 9999 stop 'ROUTINE CONVERT_PRE: ERROR'
      END

      SUBROUTINE PHGCUT(CXMN,FELD,WSAVE,Z,
     *                  MNAUF,MAUF,MANF,MAXL,MAXB,MLEVEL)

C     DIE ROUTINE F]HRT EINE TRANSFORMATION EINER
C     FELDVARIABLEN VOM PHASENRAUM IN  DEN PHYSIKALISCHEN
C     RAUM AUF KUGELKOORDINATEN DURCH
C
C     CXMN   = SPEKTRALKOEFFIZIENTEN IN DER REIHENFOLGE
C              CX00,CX01,CX11,CX02,....CXMNAUFMNAUF
C     FELD   = FELD DER METEOROLOGISCHEN VARIABLEN
C     BREITE = SINUS DER GEOGRAFISCHEN BREITEN
C
C     MNAUF    ANZAHL DER FOURIERKOEFFIZIENTEN
C     MAUF     ANZAHL DER LAENGEN UND DER FOURIERKOEFFIZIENTEN
C     MANF     ANFANG DES LAENGENBEREICHS FUER DAS GITTER,
C              AUF DAS INTERPOLIERT WERDEN SOLL
C     MAXL     ANZAHL DER FUER DAS GITTER BENUTZTEN LAENGEN
C     MAXB     ANZAHL DER FUER DAS GITTER BENOETIGTEN BREITEN
C     MLEVEL   ANZAHL DER LEVELS, DIE TRANSFORMIERT WERDEN
C
        IMPLICIT REAL*8 (A-H,O-Z)
        PARAMETER (MAXAUF=2048)

C    FELD DER FOURIERKOEFFIZIENTEN
        DIMENSION CXM(0:MAXAUF-1)

        DIMENSION HILF(MAXAUF)

C    FELD DER LEGENDREPOLYNOMEE
        REAL*8 Z(0:((MNAUF+1)*(MNAUF+2))/2,MAXB)

        DIMENSION CXMN(0:(MNAUF+1)*(MNAUF+2)-1,MLEVEL)
        REAL*8 FELD(MAXL,MAXB,MLEVEL)
      DIMENSION WSAVE(4*MAUF*2+15)

C
        IF(MAUF.LE.MNAUF) PAUSE 'LAENGENAUFLOESUNG ZU GROB'
          IF((MANF.LT.1).OR.(MAXL.LT.1).OR.
     *       (MANF.GT.MAUF).OR.(MAXL.GT.MAUF)) THEN
            PAUSE 'FALSCHER LAENGENBEREICH'
            STOP
          ENDIF

      DO 6 J=1,MAXB
          DO 6 L=1,MLEVEL
*            WRITE(*,*) L
						IF(MAUF.LT.2*MNAUF) THEN
            	CALL LEGTR(CXMN(0,L),CXM,Z(0,J),MNAUF,2*MAUF)
            	CALL FOURTR(CXM,HILF,WSAVE,MNAUF,2*MAUF,MAUF,1)
            ELSE
            	CALL LEGTR(CXMN(0,L),CXM,Z(0,J),MNAUF,MAUF)
            	CALL FOURTR(CXM,HILF,WSAVE,MNAUF,MAUF,MAUF,1)
            ENDIF
            DO 6 I=0,MAXL-1
                  IF(MANF+I.LE.MAUF) THEN
                    FELD(I+1,J,L)=HILF(MANF+I)
                  ELSE
                    FELD(I+1,J,L)=HILF(MANF-MAUF+I)
                  ENDIF
    6   CONTINUE

        RETURN
        END
C
        SUBROUTINE LEGTR(CXMN,CXM,Z,MNAUF,MAUF)
        IMPLICIT REAL*8 (A-H,O-Z)
        DIMENSION CXM(0:2*MAUF-1)
        DIMENSION CXMN(0:2*(((MNAUF+1)*MNAUF)/2+MNAUF)+1)
        REAL*8 Z(0:((MNAUF+1)*(MNAUF+2))/2)
        REAL*8 CR,CI
C
C     DIESE ROUTINE BERECHNET DIE FOURIERKOEFFIZIENTEN CXM
C
        DO 1 I=0,MNAUF
            CR=0.0
            CI=0.0
            INDEX=((I+1)*I)/2
            DO 2 J=I,MNAUF
              INDEX=INDEX+J
              CR=CR+Z(INDEX)*CXMN(2*INDEX)
              CI=CI+Z(INDEX)*CXMN(2*INDEX+1)
    2     CONTINUE
          CXM(2*I)=CR
            CXM(2*I+1)=CI
    1 CONTINUE
C       IF(CXM(1).NE.0.D0) THEN
C         WRITE(*,*) 'INVALID INPUT DATA'
C         WRITE(*,*) CXM(1)
C       ENDIF
        RETURN
        END
C
      SUBROUTINE FOURTR(CXM,FELD,WSAVE,MNAUF,FFTAUF,MAUF,ISIGN)
C     BERECHNET DIE FOURIERSUMME MIT EINEM FFT-ALGORITHMUS
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER FFTAUF
      DIMENSION CXM(0:2*FFTAUF+1)
      DIMENSION FELD(MAUF)
      DIMENSION WSAVE(*)
C
C     ANPASSEN DES FELDES DER FOURIERKOEFFIZIENTEN, UM
C     DIE FFT-ROUTINE VERWENDEN ZU KOENNEN

      DO 1 I=MNAUF+1,FFTAUF
          CXM(2*I)=0.0
          CXM(2*I+1)=0.0
    1 CONTINUE
      DO 3 I=1,MNAUF
          CXM(2*I)=2.0*CXM(2*I)
          CXM(2*I+1)=2.0*CXM(2*I+1)
    3 CONTINUE
C	Wenn MAUF<FFTAUF dann liefert FFT zuviele Gitterpunkte
C	In unserem Fall liefert sie 720 Gitterpunkte, von denen jeder
C	zweite genommen wird (Abstand 4, weil nur Realteil verwendet wird)
      CALL CFFTB(FFTAUF,CXM,WSAVE)
      DO 11 I=0,MAUF-1
      	IF(FFTAUF.GT.MAUF) THEN
        	FELD(I+1)=CXM(4*I)
        ELSE
        	FELD(I+1)=CXM(2*I)
        ENDIF
   11 CONTINUE
      RETURN
      END
C     
        SUBROUTINE PLGNDN(L,X,Z)
C
C PLGNDN BERECHNET ALLE NORMIERTEN ASSOZIIERTEN
C LEGENDREFUNKTIONEN VON P00(X) BIS PLL(X)
C UND SCHREIBT SIE IN DAS FELD Z
C DIE REIHENFOLGE IST
C P00(X),P01(X),P11(X),P02(X),P12(X),P22(X),..PLL(X)
C X IST DER COSINUS DES ZENITWINKELS ODER
C       DER SINUS DER GEOGRAFISCHEN BREITE
C
        IMPLICIT REAL*8 (A-H,O-Z)
        REAL*8 Z(0:((L+1)*(L+2))/2)
C
      Z(0)=1.0
      FACT=1.0
        POT=1.0
      SOMX2=SQRT(1.0-X*X)
        DO 14 J=0,L
          I=((J+1)*J)/2+J
        IF(J.GT.0) THEN
          FACT=FACT*FLOAT(2*J-1)/FLOAT(2*J)
            POT=POT*SOMX2
          Z(I)=SQRT(FLOAT(2*J+1)*FACT)*POT
        ENDIF
          IF(J.LT.L) THEN
          Z(I+J+1)=X*SQRT(FLOAT(4*J*J+8*J+3)/FLOAT(2*J+1))*Z(I)
          ENDIF
          I=((J+3)*(J+2))/2-2
        DO 14 K=J+2,L
            I=I+K
          Z(I)=X*SQRT(FLOAT(4*K*K-1)/FLOAT(K*K-J*J))*Z(I-K)-
     *    SQRT(FLOAT((2*K+1)*(K-J-1)*(K+J-1))/
     *    FLOAT((2*K-3)*(K*K-J*J)))*Z(I-2*K+1)
14    CONTINUE
      RETURN
      END

C
      SUBROUTINE CFFTI (N,WSAVE)
C   ******************************************************************
C
C   SUBROUTINE CFFTI(N,WSAVE)
C
C   ******************************************************************
C
C   SUBROUTINE CFFTI INITIALIZES THE ARRAY WSAVE WHICH IS USED IN
C   BOTH CFFTF AND CFFTB. THE PRIME FACTORIZATION OF N TOGETHER WITH
C   A TABULATION OF THE TRIGONOMETRIC FUNCTIONS ARE COMPUTED AND
C   STORED IN WSAVE.
C
C   INPUT PARAMETER
C
C   N       THE LENGTH OF THE SEQUENCE TO BE TRANSFORMED
C
C   OUTPUT PARAMETER
C
C   WSAVE   A WORK ARRAY WHICH MUST BE DIMENSIONED AT LEAST 4*N+15
C           THE SAME WORK ARRAY CAN BE USED FOR BOTH CFFTF AND CFFTB
C           AS LONG AS N REMAINS UNCHANGED. DIFFERENT WSAVE ARRAYS
C           ARE REQUIRED FOR DIFFERENT VALUES OF N. THE CONTENTS OF
C           WSAVE MUST NOT BE CHANGED BETWEEN CALLS OF CFFTF OR CFFTB.
C
C   ******************************************************************
      IMPLICIT NONE
      INTEGER IW1,IW2,N
      REAL*8 WSAVE
      DIMENSION       WSAVE(*)
      IF (N .EQ. 1) RETURN
      IW1 = N+N+1
      IW2 = IW1+N+N
      CALL CFFTI1 (N,WSAVE(IW1),WSAVE(IW2))
      RETURN
      END
      
      SUBROUTINE CFFTI1 (N,WA,IFAC)

      IMPLICIT REAL*8 (A-H,O-Z)

      DIMENSION       WA(*)      ,IFAC(*)    ,NTRYH(4)
      DATA NTRYH(1),NTRYH(2),NTRYH(3),NTRYH(4)/3,4,2,5/
      NL = N
      NF = 0
      J = 0
  101 J = J+1
      IF (J-4) 102,102,103
  102 NTRY = NTRYH(J)
      GO TO 104
  103 NTRY = NTRY+2
  104 NQ = NL/NTRY
      NR = NL-NTRY*NQ
      IF (NR) 101,105,101
  105 NF = NF+1
      IFAC(NF+2) = NTRY
      NL = NQ
      IF (NTRY .NE. 2) GO TO 107
      IF (NF .EQ. 1) GO TO 107
      DO 106 I=2,NF
         IB = NF-I+2
         IFAC(IB+2) = IFAC(IB+1)
  106 CONTINUE
      IFAC(3) = 2
  107 IF (NL .NE. 1) GO TO 104
      IFAC(1) = N
      IFAC(2) = NF
      TPI = 6.28318530717959D0
      ARGH = TPI/FLOAT(N)
      I = 2
      L1 = 1
      DO 110 K1=1,NF
         IP = IFAC(K1+2)
         LD = 0
         L2 = L1*IP
         IDO = N/L2
         IDOT = IDO+IDO+2
         IPM = IP-1
         DO 109 J=1,IPM
            I1 = I
            WA(I-1) = 1.0D0
            WA(I) = 0.0D0
            LD = LD+L1
            FI = 0.0D0
            ARGLD = FLOAT(LD)*ARGH
            DO 108 II=4,IDOT,2
               I = I+2
               FI = FI+1.0D0
               ARG = FI*ARGLD
               WA(I-1) = DCOS(ARG)
               WA(I) = DSIN(ARG)
  108       CONTINUE
            IF (IP .LE. 5) GO TO 109
            WA(I1-1) = WA(I-1)
            WA(I1) = WA(I)
  109    CONTINUE
         L1 = L2
  110 CONTINUE
      RETURN
      END

      SUBROUTINE CFFTB (N,C,WSAVE)
C   ******************************************************************
C
C   SUBROUTINE CFFTB(N,C,WSAVE)
C
C   ******************************************************************
C
C   SUBROUTINE CFFTB COMPUTES THE BACKWARD COMPLEX DISCRETE FOURIER
C   TRANSFORM (THE FOURIER SYNTHESIS). EQUIVALENTLY , CFFTB COMPUTES
C   A COMPLEX PERIODIC SEQUENCE FROM ITS FOURIER COEFFICIENTS.
C   THE TRANSFORM IS DEFINED BELOW AT OUTPUT PARAMETER C.
C
C   A CALL OF CFFTF FOLLOWED BY A CALL OF CFFTB WILL MULTIPLY THE
C   SEQUENCE BY N.
C
C   THE ARRAY WSAVE WHICH IS USED BY SUBROUTINE CFFTB MUST BE
C   INITIALIZED BY CALLING SUBROUTINE CFFTI(N,WSAVE).
C
C   INPUT PARAMETERS
C
C
C   N      THE LENGTH OF THE COMPLEX SEQUENCE C. THE METHOD IS
C          MORE EFFICIENT WHEN N IS THE PRODUCT OF SMALL PRIMES.
C
C   C      A COMPLEX ARRAY OF LENGTH N WHICH CONTAINS THE SEQUENCE
C
C   WSAVE   A REAL WORK ARRAY WHICH MUST BE DIMENSIONED AT LEAST 4N+15
C           IN THE PROGRAM THAT CALLS CFFTB. THE WSAVE ARRAY MUST BE
C           INITIALIZED BY CALLING SUBROUTINE CFFTI(N,WSAVE) AND A
C           DIFFERENT WSAVE ARRAY MUST BE USED FOR EACH DIFFERENT
C           VALUE OF N. THIS INITIALIZATION DOES NOT HAVE TO BE
C           REPEATED SO LONG AS N REMAINS UNCHANGED THUS SUBSEQUENT
C           TRANSFORMS CAN BE OBTAINED FASTER THAN THE FIRST.
C           THE SAME WSAVE ARRAY CAN BE USED BY CFFTF AND CFFTB.
C
C   OUTPUT PARAMETERS
C
C   C      FOR J=1,...,N
C
C              C(J)=THE SUM FROM K=1,...,N OF
C
C                    C(K)*EXP(I*(J-1)*(K-1)*2*PI/N)
C
C                          WHERE I=SQRT(-1)
C
C   WSAVE   CONTAINS INITIALIZATION CALCULATIONS WHICH MUST NOT BE
C           DESTROYED BETWEEN CALLS OF SUBROUTINE CFFTF OR CFFTB
C   ******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION       C(*)       ,WSAVE(*)
      IF (N .EQ. 1) RETURN
      IW1 = N+N+1
      IW2 = IW1+N+N
      CALL CFFTB1 (N,C,WSAVE,WSAVE(IW1),WSAVE(IW2))
      RETURN
      END
      
      SUBROUTINE CFFTB1 (N,C,CH,WA,IFAC)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION       CH(*)      ,C(*)       ,WA(*)      ,IFAC(*)
      NF = IFAC(2)
      NA = 0
      L1 = 1
      IW = 1
      DO 116 K1=1,NF
         IP = IFAC(K1+2)
         L2 = IP*L1
         IDO = N/L2
         IDOT = IDO+IDO
         IDL1 = IDOT*L1
         IF (IP .NE. 4) GO TO 103
         IX2 = IW+IDOT
         IX3 = IX2+IDOT
         IF (NA .NE. 0) GO TO 101
         CALL PASSB4 (IDOT,L1,C,CH,WA(IW),WA(IX2),WA(IX3))
         GO TO 102
  101    CALL PASSB4 (IDOT,L1,CH,C,WA(IW),WA(IX2),WA(IX3))
  102    NA = 1-NA
         GO TO 115
  103    IF (IP .NE. 2) GO TO 106
         IF (NA .NE. 0) GO TO 104
         CALL PASSB2 (IDOT,L1,C,CH,WA(IW))
         GO TO 105
  104    CALL PASSB2 (IDOT,L1,CH,C,WA(IW))
  105    NA = 1-NA
         GO TO 115
  106    IF (IP .NE. 3) GO TO 109
         IX2 = IW+IDOT
         IF (NA .NE. 0) GO TO 107
         CALL PASSB3 (IDOT,L1,C,CH,WA(IW),WA(IX2))
         GO TO 108
  107    CALL PASSB3 (IDOT,L1,CH,C,WA(IW),WA(IX2))
  108    NA = 1-NA
         GO TO 115
  109    IF (IP .NE. 5) GO TO 112
         IX2 = IW+IDOT
         IX3 = IX2+IDOT
         IX4 = IX3+IDOT
         IF (NA .NE. 0) GO TO 110
         CALL PASSB5 (IDOT,L1,C,CH,WA(IW),WA(IX2),WA(IX3),WA(IX4))
         GO TO 111
  110    CALL PASSB5 (IDOT,L1,CH,C,WA(IW),WA(IX2),WA(IX3),WA(IX4))
  111    NA = 1-NA
         GO TO 115
  112    IF (NA .NE. 0) GO TO 113
         CALL PASSB (NAC,IDOT,IP,L1,IDL1,C,C,C,CH,CH,WA(IW))
         GO TO 114
  113    CALL PASSB (NAC,IDOT,IP,L1,IDL1,CH,CH,CH,C,C,WA(IW))
  114    IF (NAC .NE. 0) NA = 1-NA
  115    L1 = L2
         IW = IW+(IP-1)*IDOT
  116 CONTINUE
      IF (NA .EQ. 0) RETURN
      N2 = N+N
      DO 117 I=1,N2
         C(I) = CH(I)
  117 CONTINUE
      RETURN
      END
      
      SUBROUTINE PASSB (NAC,IDO,IP,L1,IDL1,CC,C1,C2,CH,CH2,WA)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION       CH(IDO,L1,IP)          ,CC(IDO,IP,L1)          ,
     1                C1(IDO,L1,IP)          ,WA(*)      ,C2(IDL1,IP),
     2                CH2(IDL1,IP)
      IDOT = IDO/2
      NT = IP*IDL1
      IPP2 = IP+2
      IPPH = (IP+1)/2
      IDP = IP*IDO
C
      IF (IDO .LT. L1) GO TO 106
      DO 103 J=2,IPPH
         JC = IPP2-J
         DO 102 K=1,L1
            DO 101 I=1,IDO
               CH(I,K,J) = CC(I,J,K)+CC(I,JC,K)
               CH(I,K,JC) = CC(I,J,K)-CC(I,JC,K)
  101       CONTINUE
  102    CONTINUE
  103 CONTINUE
      DO 105 K=1,L1
         DO 104 I=1,IDO
            CH(I,K,1) = CC(I,1,K)
  104    CONTINUE
  105 CONTINUE
      GO TO 112
  106 DO 109 J=2,IPPH
         JC = IPP2-J
         DO 108 I=1,IDO
            DO 107 K=1,L1
               CH(I,K,J) = CC(I,J,K)+CC(I,JC,K)
               CH(I,K,JC) = CC(I,J,K)-CC(I,JC,K)
  107       CONTINUE
  108    CONTINUE
  109 CONTINUE
      DO 111 I=1,IDO
         DO 110 K=1,L1
            CH(I,K,1) = CC(I,1,K)
  110    CONTINUE
  111 CONTINUE
  112 IDL = 2-IDO
      INC = 0
      DO 116 L=2,IPPH
         LC = IPP2-L
         IDL = IDL+IDO
         DO 113 IK=1,IDL1
            C2(IK,L) = CH2(IK,1)+WA(IDL-1)*CH2(IK,2)
            C2(IK,LC) = WA(IDL)*CH2(IK,IP)
  113    CONTINUE
         IDLJ = IDL
         INC = INC+IDO
         DO 115 J=3,IPPH
            JC = IPP2-J
            IDLJ = IDLJ+INC
            IF (IDLJ .GT. IDP) IDLJ = IDLJ-IDP
            WAR = WA(IDLJ-1)
            WAI = WA(IDLJ)
            DO 114 IK=1,IDL1
               C2(IK,L) = C2(IK,L)+WAR*CH2(IK,J)
               C2(IK,LC) = C2(IK,LC)+WAI*CH2(IK,JC)
  114       CONTINUE
  115    CONTINUE
  116 CONTINUE
      DO 118 J=2,IPPH
         DO 117 IK=1,IDL1
            CH2(IK,1) = CH2(IK,1)+CH2(IK,J)
  117    CONTINUE
  118 CONTINUE
      DO 120 J=2,IPPH
         JC = IPP2-J
         DO 119 IK=2,IDL1,2
            CH2(IK-1,J) = C2(IK-1,J)-C2(IK,JC)
            CH2(IK-1,JC) = C2(IK-1,J)+C2(IK,JC)
            CH2(IK,J) = C2(IK,J)+C2(IK-1,JC)
            CH2(IK,JC) = C2(IK,J)-C2(IK-1,JC)
  119    CONTINUE
  120 CONTINUE
      NAC = 1
      IF (IDO .EQ. 2) RETURN
      NAC = 0
      DO 121 IK=1,IDL1
         C2(IK,1) = CH2(IK,1)
  121 CONTINUE
      DO 123 J=2,IP
         DO 122 K=1,L1
            C1(1,K,J) = CH(1,K,J)
            C1(2,K,J) = CH(2,K,J)
  122    CONTINUE
  123 CONTINUE
      IF (IDOT .GT. L1) GO TO 127
      IDIJ = 0
      DO 126 J=2,IP
         IDIJ = IDIJ+2
         DO 125 I=4,IDO,2
            IDIJ = IDIJ+2
            DO 124 K=1,L1
               C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)-WA(IDIJ)*CH(I,K,J)
               C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)+WA(IDIJ)*CH(I-1,K,J)
  124       CONTINUE
  125    CONTINUE
  126 CONTINUE
      RETURN
  127 IDJ = 2-IDO
      DO 130 J=2,IP
         IDJ = IDJ+IDO
         DO 129 K=1,L1
            IDIJ = IDJ
            DO 128 I=4,IDO,2
               IDIJ = IDIJ+2
               C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)-WA(IDIJ)*CH(I,K,J)
               C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)+WA(IDIJ)*CH(I-1,K,J)
  128       CONTINUE
  129    CONTINUE
  130 CONTINUE
      RETURN
      END
      
      SUBROUTINE PASSB2 (IDO,L1,CC,CH,WA1)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION       CC(IDO,2,L1)           ,CH(IDO,L1,2)           ,
     1                WA1(*)
      IF (IDO .GT. 2) GO TO 102
      DO 101 K=1,L1
         CH(1,K,1) = CC(1,1,K)+CC(1,2,K)
         CH(1,K,2) = CC(1,1,K)-CC(1,2,K)
         CH(2,K,1) = CC(2,1,K)+CC(2,2,K)
         CH(2,K,2) = CC(2,1,K)-CC(2,2,K)
  101 CONTINUE
      RETURN
  102 DO 104 K=1,L1
         DO 103 I=2,IDO,2
            CH(I-1,K,1) = CC(I-1,1,K)+CC(I-1,2,K)
            TR2 = CC(I-1,1,K)-CC(I-1,2,K)
            CH(I,K,1) = CC(I,1,K)+CC(I,2,K)
            TI2 = CC(I,1,K)-CC(I,2,K)
            CH(I,K,2) = WA1(I-1)*TI2+WA1(I)*TR2
            CH(I-1,K,2) = WA1(I-1)*TR2-WA1(I)*TI2
  103    CONTINUE
  104 CONTINUE
      RETURN
      END
      
      SUBROUTINE PASSB3 (IDO,L1,CC,CH,WA1,WA2)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION       CC(IDO,3,L1)           ,CH(IDO,L1,3)           ,
     1                WA1(*)     ,WA2(*)
      DATA TAUR,TAUI /-.5D0,.866025403784439D0/
      IF (IDO .NE. 2) GO TO 102
      DO 101 K=1,L1
         TR2 = CC(1,2,K)+CC(1,3,K)
         CR2 = CC(1,1,K)+TAUR*TR2
         CH(1,K,1) = CC(1,1,K)+TR2
         TI2 = CC(2,2,K)+CC(2,3,K)
         CI2 = CC(2,1,K)+TAUR*TI2
         CH(2,K,1) = CC(2,1,K)+TI2
         CR3 = TAUI*(CC(1,2,K)-CC(1,3,K))
         CI3 = TAUI*(CC(2,2,K)-CC(2,3,K))
         CH(1,K,2) = CR2-CI3
         CH(1,K,3) = CR2+CI3
         CH(2,K,2) = CI2+CR3
         CH(2,K,3) = CI2-CR3
  101 CONTINUE
      RETURN
  102 DO 104 K=1,L1
         DO 103 I=2,IDO,2
            TR2 = CC(I-1,2,K)+CC(I-1,3,K)
            CR2 = CC(I-1,1,K)+TAUR*TR2
            CH(I-1,K,1) = CC(I-1,1,K)+TR2
            TI2 = CC(I,2,K)+CC(I,3,K)
            CI2 = CC(I,1,K)+TAUR*TI2
            CH(I,K,1) = CC(I,1,K)+TI2
            CR3 = TAUI*(CC(I-1,2,K)-CC(I-1,3,K))
            CI3 = TAUI*(CC(I,2,K)-CC(I,3,K))
            DR2 = CR2-CI3
            DR3 = CR2+CI3
            DI2 = CI2+CR3
            DI3 = CI2-CR3
            CH(I,K,2) = WA1(I-1)*DI2+WA1(I)*DR2
            CH(I-1,K,2) = WA1(I-1)*DR2-WA1(I)*DI2
            CH(I,K,3) = WA2(I-1)*DI3+WA2(I)*DR3
            CH(I-1,K,3) = WA2(I-1)*DR3-WA2(I)*DI3
  103    CONTINUE
  104 CONTINUE
      RETURN
      END
      
      SUBROUTINE PASSB4 (IDO,L1,CC,CH,WA1,WA2,WA3)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION       CC(IDO,4,L1)           ,CH(IDO,L1,4)           ,
     1                WA1(*)     ,WA2(*)     ,WA3(*)
      IF (IDO .NE. 2) GO TO 102
      DO 101 K=1,L1
         TI1 = CC(2,1,K)-CC(2,3,K)
         TI2 = CC(2,1,K)+CC(2,3,K)
         TR4 = CC(2,4,K)-CC(2,2,K)
         TI3 = CC(2,2,K)+CC(2,4,K)
         TR1 = CC(1,1,K)-CC(1,3,K)
         TR2 = CC(1,1,K)+CC(1,3,K)
         TI4 = CC(1,2,K)-CC(1,4,K)
         TR3 = CC(1,2,K)+CC(1,4,K)
         CH(1,K,1) = TR2+TR3
         CH(1,K,3) = TR2-TR3
         CH(2,K,1) = TI2+TI3
         CH(2,K,3) = TI2-TI3
         CH(1,K,2) = TR1+TR4
         CH(1,K,4) = TR1-TR4
         CH(2,K,2) = TI1+TI4
         CH(2,K,4) = TI1-TI4
  101 CONTINUE
      RETURN
  102 DO 104 K=1,L1
         DO 103 I=2,IDO,2
            TI1 = CC(I,1,K)-CC(I,3,K)
            TI2 = CC(I,1,K)+CC(I,3,K)
            TI3 = CC(I,2,K)+CC(I,4,K)
            TR4 = CC(I,4,K)-CC(I,2,K)
            TR1 = CC(I-1,1,K)-CC(I-1,3,K)
            TR2 = CC(I-1,1,K)+CC(I-1,3,K)
            TI4 = CC(I-1,2,K)-CC(I-1,4,K)
            TR3 = CC(I-1,2,K)+CC(I-1,4,K)
            CH(I-1,K,1) = TR2+TR3
            CR3 = TR2-TR3
            CH(I,K,1) = TI2+TI3
            CI3 = TI2-TI3
            CR2 = TR1+TR4
            CR4 = TR1-TR4
            CI2 = TI1+TI4
            CI4 = TI1-TI4
            CH(I-1,K,2) = WA1(I-1)*CR2-WA1(I)*CI2
            CH(I,K,2) = WA1(I-1)*CI2+WA1(I)*CR2
            CH(I-1,K,3) = WA2(I-1)*CR3-WA2(I)*CI3
            CH(I,K,3) = WA2(I-1)*CI3+WA2(I)*CR3
            CH(I-1,K,4) = WA3(I-1)*CR4-WA3(I)*CI4
            CH(I,K,4) = WA3(I-1)*CI4+WA3(I)*CR4
  103    CONTINUE
  104 CONTINUE
      RETURN
      END
      
      SUBROUTINE PASSB5 (IDO,L1,CC,CH,WA1,WA2,WA3,WA4)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION       CC(IDO,5,L1)           ,CH(IDO,L1,5)           ,
     1                WA1(*)     ,WA2(*)     ,WA3(*)     ,WA4(*)
      DATA TR11,TI11,TR12,TI12 /.309016994374947D0,.951056516295154D0,
     1-.809016994374947D0,.587785252292473D0/
      IF (IDO .NE. 2) GO TO 102
      DO 101 K=1,L1
         TI5 = CC(2,2,K)-CC(2,5,K)
         TI2 = CC(2,2,K)+CC(2,5,K)
         TI4 = CC(2,3,K)-CC(2,4,K)
         TI3 = CC(2,3,K)+CC(2,4,K)
         TR5 = CC(1,2,K)-CC(1,5,K)
         TR2 = CC(1,2,K)+CC(1,5,K)
         TR4 = CC(1,3,K)-CC(1,4,K)
         TR3 = CC(1,3,K)+CC(1,4,K)
         CH(1,K,1) = CC(1,1,K)+TR2+TR3
         CH(2,K,1) = CC(2,1,K)+TI2+TI3
         CR2 = CC(1,1,K)+TR11*TR2+TR12*TR3
         CI2 = CC(2,1,K)+TR11*TI2+TR12*TI3
         CR3 = CC(1,1,K)+TR12*TR2+TR11*TR3
         CI3 = CC(2,1,K)+TR12*TI2+TR11*TI3
         CR5 = TI11*TR5+TI12*TR4
         CI5 = TI11*TI5+TI12*TI4
         CR4 = TI12*TR5-TI11*TR4
         CI4 = TI12*TI5-TI11*TI4
         CH(1,K,2) = CR2-CI5
         CH(1,K,5) = CR2+CI5
         CH(2,K,2) = CI2+CR5
         CH(2,K,3) = CI3+CR4
         CH(1,K,3) = CR3-CI4
         CH(1,K,4) = CR3+CI4
         CH(2,K,4) = CI3-CR4
         CH(2,K,5) = CI2-CR5
  101 CONTINUE
      RETURN
  102 DO 104 K=1,L1
         DO 103 I=2,IDO,2
            TI5 = CC(I,2,K)-CC(I,5,K)
            TI2 = CC(I,2,K)+CC(I,5,K)
            TI4 = CC(I,3,K)-CC(I,4,K)
            TI3 = CC(I,3,K)+CC(I,4,K)
            TR5 = CC(I-1,2,K)-CC(I-1,5,K)
            TR2 = CC(I-1,2,K)+CC(I-1,5,K)
            TR4 = CC(I-1,3,K)-CC(I-1,4,K)
            TR3 = CC(I-1,3,K)+CC(I-1,4,K)
            CH(I-1,K,1) = CC(I-1,1,K)+TR2+TR3
            CH(I,K,1) = CC(I,1,K)+TI2+TI3
            CR2 = CC(I-1,1,K)+TR11*TR2+TR12*TR3
            CI2 = CC(I,1,K)+TR11*TI2+TR12*TI3
            CR3 = CC(I-1,1,K)+TR12*TR2+TR11*TR3
            CI3 = CC(I,1,K)+TR12*TI2+TR11*TI3
            CR5 = TI11*TR5+TI12*TR4
            CI5 = TI11*TI5+TI12*TI4
            CR4 = TI12*TR5-TI11*TR4
            CI4 = TI12*TI5-TI11*TI4
            DR3 = CR3-CI4
            DR4 = CR3+CI4
            DI3 = CI3+CR4
            DI4 = CI3-CR4
            DR5 = CR2+CI5
            DR2 = CR2-CI5
            DI5 = CI2-CR5
            DI2 = CI2+CR5
            CH(I-1,K,2) = WA1(I-1)*DR2-WA1(I)*DI2
            CH(I,K,2) = WA1(I-1)*DI2+WA1(I)*DR2
            CH(I-1,K,3) = WA2(I-1)*DR3-WA2(I)*DI3
            CH(I,K,3) = WA2(I-1)*DI3+WA2(I)*DR3
            CH(I-1,K,4) = WA3(I-1)*DR4-WA3(I)*DI4
            CH(I,K,4) = WA3(I-1)*DI4+WA3(I)*DR4
            CH(I-1,K,5) = WA4(I-1)*DR5-WA4(I)*DI5
            CH(I,K,5) = WA4(I-1)*DI5+WA4(I)*DR5
  103    CONTINUE
  104 CONTINUE
      RETURN
      END
      
      SUBROUTINE INOUT (  FELD,  NI, NJ,  NK,  FILEN, ELEMEN, EINHEI,
     *                DATEN, LS, IGX, IGY, ISX,ISY)

      IMPLICIT REAL*8 (A-H,O-Z)

	REAL*8 FELD(NI,NJ,NK),G
	
	REAL*8 RMS,MW,SIG
	
	CHARACTER*1 A,A1,LS
	CHARACTER*2 A2
	CHARACTER*5 Z,SMEAN,SRMS,SVAR
	CHARACTER*8 TERMIN
	CHARACTER*12 FILEN
	CHARACTER*15 STAT
	CHARACTER*70 EINHEI
	CHARACTER*80 ELEMEN,DATEN
	LOGICAL EX


	WRITE(*,*) FILEN
	
	IF(LS.EQ.'S') THEN
	
	  OPEN(1,FILE=FILEN)
	  
	  Z='10**('
	  A='x'
	  A1=' '
	  A2=') '
	  STAT='STATISTICS:'
	  SMEAN='MEAN='
	  SRMS=' RMS='
	  SVAR=' VAR='
	  TERMIN=FILEN(5:12)
  
	  G=9999.
	  IS=IA(FELD,NI,NJ,NK,G)
  
	  CALL STATIS(NI,NJ,NK,FELD,RMS,MW,SIG)
C
	FAKT=10.0**IS
	
	RMS = RMS*FAKT
	MW  = MW *FAKT
	SIG = SIG*FAKT
	
      WRITE(1,'(A80)') ELEMEN
      WRITE(1,'(A5,I3,A2,A70)') Z,-IS,A2,EINHEI
      WRITE(1,'(A8)') TERMIN
      WRITE(1,'(I3,2(A1,I3),2X,I3,A1,I3,2X,I3,A1,I3)')
     *		NI,A,NJ,A,NK,IGX,A,IGY,ISX,A,ISY
      WRITE(1,'(A80)') DATEN
      WRITE (1,'(A15,A5,F8.2,A1,A5,F8.2,A1,A5,F8.2)') STAT,SMEAN,
     *		MW,A1,SRMS,RMS,A1,SVAR,SIG
      WRITE (*,'(A15,A5,F8.2,A1,A5,F8.2,A1,A5,F8.2)') STAT,SMEAN,
     *		MW,A1,SRMS,RMS,A1,SVAR,SIG
      WRITE (1,'(A1)') A1
	  
      DO 21 K=1,NK
        DO 20 J=1,NJ
          WRITE(1,'(10F8.2)') (FELD(I,J,K)*FAKT,I=1,NI)
20      CONTINUE
	  WRITE(1,'(A1)') A1
21    CONTINUE
      
      ELSEIF(LS.EQ.'L') THEN
      
	INQUIRE(FILE=FILEN,EXIST=EX)
	  IF(.NOT.EX) THEN
	  	WRITE(*,*) 'Termin oder Filename falsch'
	  	RETURN
	ELSE
	  OPEN(1,FILE=FILEN)	
	ENDIF

	  READ (1,'(A80)') ELEMEN
	  READ(1,'(A5,I3,A2,A70)')  Z,IS,A2,EINHEI
	  WRITE(*,'(A5,I3,A2,A70)')  Z,IS,A2,EINHEI
	  READ(1,'(A8)') TERMIN
	  READ(1,'(I3,2(A1,I3),2X,I3,A1,I3,2X,I3,A1,I3)')
     *		MI,A,MJ,A,MK,IGX,A,IGY,ISX,A,ISY
        READ(1,'(A80)') DATEN	
        READ (1,'(A15,A5,F8.2,A1,A5,F8.2,A1,A5,F8.2)') STAT,SMEAN,
     *		MW,A1,SRMS,RMS,A1,SVAR,SIG
        WRITE (*,'(A15,A5,F8.2,A1,A5,F8.2,A1,A5,F8.2)') STAT,SMEAN,
     *		MW,A1,SRMS,RMS,A1,SVAR,SIG
        READ (1,'(A1)') A1
	  IF(TERMIN.NE.FILEN(5:12)) THEN
	  	WRITE(*,*) 'Termin oder Filename falsch'
	  	RETURN
	  ENDIF
	  IF((MI.GT.IGX).OR.(MJ.GT.IGY).OR.(MK.GT.NK)) THEN
	  	WRITE(*,*) 'Falsche Felddimensionen'
	  	RETURN
	  ENDIF

	  DO 132 K=1,MK
	    DO 131 J=1,NJ
	      READ(1,'(10F8.2)') (FELD(I,J,K),I=1,NI)
131        CONTINUE
          READ(1,'(A1)') A1
132      CONTINUE

	  FAKT=10.0**IS
	  
	  DO 140 K=1,MK
	    DO 140 J=1,NJ
		DO 140 I=1,NI
140            FELD(I,J,K)=FELD(I,J,K)*FAKT

      ELSE
        WRITE(*,*) ' FALSCHER PARAMETER FUER LS !'
      ENDIF

      CLOSE(1)

      RETURN
      END

************************************************************************
      INTEGER FUNCTION IA (FIELD1,NI,NJ,NK,G)

      IMPLICIT NONE
      INTEGER NI,NJ,NK,I,J,K
      REAL*8 FIELD1(NI,NJ,NK)
      REAL*8 G
      REAL*8 RMIN,RMAX,XMAX,A,A1,A2

      RMAX=FIELD1(1,1,1)
      RMIN=FIELD1(1,1,1)
      
      DO 100 K=1,NK
      DO 100 J=1,NJ
        DO 100 I=1,NI
          IF (FIELD1(I,J,K).GT.RMAX)RMAX=FIELD1(I,J,K)
          IF (FIELD1(I,J,K).LT.RMIN)RMIN=FIELD1(I,J,K)
100   CONTINUE

      IF (ABS(RMIN).GT.RMAX.OR.ABS(RMIN).EQ.RMAX) THEN
      XMAX=ABS(RMIN)
      ELSE
      XMAX=RMAX
      ENDIF
      
      IF (XMAX.EQ.0) THEN
      IA = 0
      RETURN
      ENDIF
      
      A1=DLOG10 ((G/10.d0)/XMAX)
      A2=DLOG10 ( G/XMAX )
      A=DMAX1 (A1,A2)
      
      IF (A.GT.0) IA=INT(A)
      IF (A.LT.0) IA=INT(A-1.0)
      
      RETURN
      END
      
      SUBROUTINE STATIS (NI,NJ,NK,PHI,RMS,MW,SIG)
	IMPLICIT REAL*8 (A-H,O-Z)

      REAL*8 PHI(NI,NJ,NK),SIG,MW,RMS,P
 
      N=NI*NJ*NK
 
      RMS=0.
	MW=0.
 
      DO 10 I=1,NI
      DO 10 J=1,NJ
        DO 10 K=1,NK
	      P=PHI(I,J,K)
          RMS=RMS+P*P
 		MW=MW+P
10    CONTINUE

      RMS=SQRT(RMS/N)
	MW=MW/N
	
	IF(RMS*RMS-MW*MW.LT.0.) THEN	  
	  SIG=0.0
	ELSE
	  SIG=SQRT(RMS*RMS-MW*MW)
	ENDIF
 
      RETURN
      END
      
