      subroutine writeout_pre(u,v,eta,t,qa,psurf,sd,msl,tcc,u10,v10,
     &                        t2,d2,lsp,cp,sshf,ssr,ewss,nsss,jsec0,
     &                        jsec1,jsec2,jsec3,jsec4,ysec2,ysec3,
     &                        zsec4,jpunp1,inbuff,jpack)
************************************************************************
*                                                                      *
*                   AUTHOR: G. WOTAWA, 1994-05-04                      *
*                                                                      *
*                   PREPARATION OF INPUT FILES FOR POP MODEL           *
*                   METEOROLOGICAL PREPROCESSOR                        *
*                                                                      *
************************************************************************
*                                                                      *
* DESCRIPTION: This subroutine creates the meteorological data files   *
*              needed by pop model meteorological preprocessor (ECMWF  *
*              grib code). It is a part of routine PRECONVERT.         *
*              Meteorological data are available on a regular lamda    *
*              phi grid.                                               *
*                                                                      *
* METEOROLOGICAL DATA WRITTEN TO OUTPUT FILE:                          *
*                                                                      *
*              T,U,V,Q, surface data, deaccumulated flux data:         *
*              Extracted from MARS on a regular lamda-phi grid  with   *
*              with given specifications (resolution, domain).         *
*                                                                      *
*              Psurf, ETAPOINT: Calculated by program PRECONVERT on    *
*              the lamda-phi grid.                                     *
*                                                                      *
* REMARK:      ETAPOINT calculated by PRECONVERT is not the total time *
*              derivation of ECMWF vertical coordinate. It is the      *
*              total time derivation of vertical coordinate eta        *
*              multiplied by the derivation of pressure p in eta       *
*              direction (dp/d(eta)). Therefore, ETAPOINT is given in  *
*              Pascal per second.                                      *
*                                                                      *
************************************************************************
*                                                                      *
*              NAME OF OUTPUT FILE: fort.15                            *
*                                                                      *
************************************************************************
      include 'includepar'

      integer i,j,k,l,lunit,jout,ierr,iword,levdiff,lenght
      INTEGER JPACK,JPUNP1
      INTEGER JSEC0(2),ISEC0(2)
      INTEGER JSEC1(56),ISEC1(56)
      INTEGER JSEC2(22+MAXL+MAXB),ISEC2(22+MAXL+MAXB)
      INTEGER JSEC3(2),ISEC3(2)
      INTEGER JSEC4(64),ISEC4(64)
      INTEGER INBUFF(JPACK)

      REAL T(maxl,maxb,MLEVEL),U(maxl,maxb,MLEVEL),V(maxl,maxb,MLEVEL)
      REAL ETA(maxl,maxb,MLEVEL),PSURF(maxl,maxb),QA(maxl,maxb,MLEVEL)

      REAL SD(maxl,maxb),MSL(maxl,maxb),TCC(maxl,maxb),U10(maxl,maxb)
      REAL V10(maxl,maxb),T2(maxl,maxb),D2(maxl,maxb)
      REAL LSP(maxl,maxb),CP(maxl,maxb),SSHF(maxl,maxb)
      REAL SSR(maxl,maxb),EWSS(maxl,maxb),NSSS(maxl,maxb)
      REAL ORO(MAXL,MAXB),LSM(MAXL,MAXB),SDOR(MAXL,MAXB),VEG(MAXL,MAXB)
      REAL SR(MAXL,MAXB),LCC(MAXL,MAXB),MCC(MAXL,MAXB),HCC(MAXL,MAXB)
      REAL SKT(MAXL,MAXB),STL1(MAXL,MAXB),SWL1(MAXL,MAXB)
 
      REAL YSEC2(60+2*MLEVEL),ZSEC2(60+2*MLEVEL)
      REAL YSEC3(2),ZSEC3(2)
      REAL ZSEC4(JPUNP1)

      CHARACTER*1 YOPER

* READ ADDITIONAL DATA

      call PBOPEN(lunit,'OROLSM','r',ierr)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING OROLSM'
10    call PBGRIB(LUNIT,INBUFF,JPACK*8,LENGHT,IERR)
      if(ierr.eq.-1) goto 30    ! EOF DETECTED
      if(ierr.lt.-1) stop 'SUBROUTINE PBGRIB: ERROR READING OROLSM'
      YOPER = 'D'
      IERR = 1
      CALL GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      IF (IERR.NE.0) STOP 'SUBROUTINE GRIBEX: CONVERSION ERROR OROG.'
      L=1
      DO 15 J=1,MAXB
      DO 15 I=1,MAXL
         IF (ISEC1(6).eq.129) ORO (I,J)=ZSEC4(L)  ! OROGRAPHY
         IF (ISEC1(6).eq.172) LSM (I,J)=ZSEC4(L)  ! LAND-SEA-MASK
         IF (ISEC1(6).eq.160) SDOR(I,J)=ZSEC4(L)  ! STD.DEV. OF OROG.
         IF (ISEC1(6).eq.199) VEG (I,J)=ZSEC4(L)  ! % OF VEGETATION
         IF (ISEC1(6).eq.173) SR  (I,J)=ZSEC4(L)  ! SURFACE ROUGHNESS
         IF (ISEC1(6).eq.186) LCC (I,J)=ZSEC4(L)  ! LOW CLOUD COVER
         IF (ISEC1(6).eq.187) MCC (I,J)=ZSEC4(L)  ! MEDIUM CLOUD COVER
         IF (ISEC1(6).eq.188) HCC (I,J)=ZSEC4(L)  ! HIGH CLOUD COVER
         IF (ISEC1(6).eq.235) SKT (I,J)=ZSEC4(L)  ! SKIN TEMPERATURE
         IF (ISEC1(6).eq.139) STL1(I,J)=ZSEC4(L)  ! SOIL TEMP. LEV. 1
         IF (ISEC1(6).eq.140) SWL1(I,J)=ZSEC4(L)  ! SOIL WETN. LEV. 1
         L=L+1
15    CONTINUE
      GO TO 10
30    CALL PBCLOSE(LUNIT,IERR)

* CHECK LEVELS TO BE WRIITEN TO OUTPUT FILES

      if(levout.gt.mlevel)
     & stop ' CAN NOT WRITE OUT MORE THAN mlevel LEVELS'
      if(levout.le.0) stop 'CAN NOT WRITE OUT ZERO LEVELS'
      levdiff=mlevel-levout+1
*
* CREATE OUTPUT FILE "fort.15"
*

      call PBOPEN(lunit,'fort.15','a',ierr)
*
* WRITE U TO GRIB FILE
*
      JSEC1(6)=131

      do 100 k=levdiff,mlevel
         JSEC1(8)=k
         do 50 i=1,maxl
            do 50 j=1,maxb
50             zsec4((j-1)*maxl+i)=U(i,j,k)
*
         YOPER='C'
         CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     &                ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
         jout=jsec0(1)
         call PBWRITE(lunit,INBUFF,jout,ierr)
100   continue
*
* WRITE V TO GRIB FILE
*
      JSEC1(6)=132

      do 200 k=levdiff,mlevel
         JSEC1(8)=k
         do 150 i=1,maxl
            do 150 j=1,maxb
150            zsec4((j-1)*maxl+i)=V(i,j,k)
*
         YOPER='C'
         CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     &                ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
         jout=jsec0(1)
         call PBWRITE(lunit,INBUFF,jout,ierr)
200   continue
*
* WRITE ETAPOINT TO GRIB FILE
*
      JSEC1(6)=135

      do 300 k=max(levdiff-1,1),mlevel
         JSEC1(8)=k
         do 250 i=1,maxl
            do 250 j=1,maxb
250            zsec4((j-1)*maxl+i)=ETA(i,j,k)
*
         YOPER='C'
         CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     &                ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
         jout=jsec0(1)
         call PBWRITE(lunit,INBUFF,jout,ierr)
300   continue
*
* WRITE T TO GRIB FILE
*
      JSEC1(6)=130

      do 400 k=levdiff,mlevel
         JSEC1(8)=k
         do 350 i=1,maxl
            do 350 j=1,maxb
350            zsec4((j-1)*maxl+i)=T(i,j,k)
*
         YOPER='C'
         CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     &                ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
         jout=jsec0(1)
         call PBWRITE(lunit,INBUFF,jout,ierr)
400   continue
*
* WRITE PSURF TO GRIB FILE
*
      JSEC1(6)=134
      JSEC1(8)=1 
 
      do 500 i=1,maxl
         do 500 j=1,maxb
500         zsec4((j-1)*maxl+i)=PSURF(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE Q TO GRIB FILE
*
      JSEC1(6)=133

      do 600 k=levdiff,mlevel
         JSEC1(8)=k
         do 550 i=1,maxl
            do 550 j=1,maxb
550            zsec4((j-1)*maxl+i)=QA(i,j,k)
*
         YOPER='C'
         CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     &                ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
         jout=jsec0(1)
         call PBWRITE(lunit,INBUFF,jout,ierr)
600   continue
*
* WRITE SD TO GRIB FILE
*
      JSEC1(6)=141
      JSEC1(8)=0
 
      do 700 i=1,maxl
         do 700 j=1,maxb
           if(SD(I,J).lt.0.) then
             zsec4((j-1)*maxl+i)=0.0
           else
             zsec4((j-1)*maxl+i)=SD(i,j)
           endif
700   continue
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE MSL TO GRIB FILE
*
      JSEC1(6)=151
      JSEC1(8)=0
 
      do 800 i=1,maxl
         do 800 j=1,maxb
800         zsec4((j-1)*maxl+i)=MSL(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE TCC TO GRIB FILE
*
      JSEC1(6)=164
      JSEC1(8)=0
 
      do 900 i=1,maxl
         do 900 j=1,maxb
900         zsec4((j-1)*maxl+i)=TCC(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE U10 TO GRIB FILE
*
      JSEC1(6)=165
      JSEC1(8)=0
 
      do 1000 i=1,maxl
         do 1000 j=1,maxb
1000        zsec4((j-1)*maxl+i)=U10(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE V10 TO GRIB FILE
*
      JSEC1(6)=166
      JSEC1(8)=0
 
      do 1100 i=1,maxl
         do 1100 j=1,maxb
1100        zsec4((j-1)*maxl+i)=V10(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE T2 TO GRIB FILE
*
      JSEC1(6)=167
      JSEC1(8)=0
 
      do 1200 i=1,maxl
         do 1200 j=1,maxb
1200        zsec4((j-1)*maxl+i)=T2(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE D2 TO GRIB FILE
*
      JSEC1(6)=168
      JSEC1(8)=0
 
      do 1300 i=1,maxl
         do 1300 j=1,maxb
1300        zsec4((j-1)*maxl+i)=D2(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE LSP TO GRIB FILE
*
      JSEC1(6)=142
      JSEC1(8)=0
 
      do 1400 i=1,maxl
         do 1400 j=1,maxb
1400        zsec4((j-1)*maxl+i)=LSP(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE CP TO GRIB FILE
*
      JSEC1(6)=143
      JSEC1(8)=0
 
      do 1500 i=1,maxl
         do 1500 j=1,maxb
1500        zsec4((j-1)*maxl+i)=CP(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE SSHF TO GRIB FILE
*
      JSEC1(6)=146
      JSEC1(8)=0
 
      do 1600 i=1,maxl
         do 1600 j=1,maxb
1600        zsec4((j-1)*maxl+i)=SSHF(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE SSR TO GRIB FILE
*
      JSEC1(6)=176
      JSEC1(8)=0
 
      do 1700 i=1,maxl
         do 1700 j=1,maxb
1700        zsec4((j-1)*maxl+i)=SSR(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE EWSS TO GRIB FILE
*
      JSEC1(6)=180
      JSEC1(8)=0
 
      do 1800 i=1,maxl
         do 1800 j=1,maxb
1800        zsec4((j-1)*maxl+i)=EWSS(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE NSSS TO GRIB FILE
*
      JSEC1(6)=181
      JSEC1(8)=0
 
      do 1900 i=1,maxl
         do 1900 j=1,maxb
1900        zsec4((j-1)*maxl+i)=NSSS(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE ORO TO GRIB FILE
*
      JSEC1(6)=129
      JSEC1(8)=0
 
      do 2000 i=1,maxl
         do 2000 j=1,maxb
2000        zsec4((j-1)*maxl+i)=ORO(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE LSM TO GRIB FILE
*
      JSEC1(6)=172
      JSEC1(8)=0
 
      do 2100 i=1,maxl
         do 2100 j=1,maxb
2100        zsec4((j-1)*maxl+i)=LSM(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE SDOR TO GRIB FILE
*
      JSEC1(6)=160
      JSEC1(8)=0
 
      do 2200 i=1,maxl
         do 2200 j=1,maxb
2200        zsec4((j-1)*maxl+i)=SDOR(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE VEG TO GRIB FILE
*
      JSEC1(6)=199
      JSEC1(8)=0
 
      do 2300 i=1,maxl
         do 2300 j=1,maxb
2300        zsec4((j-1)*maxl+i)=VEG(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE SR TO GRIB FILE
*
      JSEC1(6)=173
      JSEC1(8)=0
 
      do 2400 i=1,maxl
         do 2400 j=1,maxb
2400        zsec4((j-1)*maxl+i)=SR(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE LCC TO GRIB FILE
*
      JSEC1(6)=186
      JSEC1(8)=0
 
      do 2500 i=1,maxl
         do 2500 j=1,maxb
2500        zsec4((j-1)*maxl+i)=LCC(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE MCC TO GRIB FILE
*
      JSEC1(6)=187
      JSEC1(8)=0
 
      do 2600 i=1,maxl
         do 2600 j=1,maxb
2600        zsec4((j-1)*maxl+i)=MCC(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE HCC TO GRIB FILE
*
      JSEC1(6)=188
      JSEC1(8)=0
 
      do 2700 i=1,maxl
         do 2700 j=1,maxb
2700        zsec4((j-1)*maxl+i)=HCC(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE SKT TO GRIB FILE
*
      JSEC1(6)=235
      JSEC1(8)=0
 
      do 2800 i=1,maxl
         do 2800 j=1,maxb
2800        zsec4((j-1)*maxl+i)=SKT(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE STL1 TO GRIB FILE
*
      JSEC1(6)=139
      JSEC1(8)=0
 
      do 2900 i=1,maxl
         do 2900 j=1,maxb
2900        zsec4((j-1)*maxl+i)=STL1(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* WRITE SWL1 TO GRIB FILE
*
      JSEC1(6)=140
      JSEC1(8)=0
 
      do 3000 i=1,maxl
         do 3000 j=1,maxb
3000        zsec4((j-1)*maxl+i)=SWL1(i,j)
*
      YOPER='C'
      CALL GRIBEX (JSEC0,JSEC1,JSEC2,YSEC2,JSEC3,YSEC3,JSEC4,
     C             ZSEC4,JPUNP1,INBUFF,JPACK,IWORD,YOPER,IERR)
      jout=jsec0(1)
      call PBWRITE(lunit,INBUFF,jout,ierr)
*
* CLOSE GRIB FILE
*
      call PBCLOSE(lunit,ierr)

      RETURN
      END
